/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

#include <lib-ui/lib-ui.h>

#include <termios.h>
#ifdef HAVE_SYS_IOCTL_H
#  include <sys/ioctl.h>
#elif defined(HAVE_IOCTL_H)
#  include <ioctl.h>
#endif

WINDOW *sidebar_w, *main_w;

void mutt_need_hard_redraw(void)
{
    keypad(main_w, true);
    clearok(main_w, true);
    set_option(OPTNEEDREDRAW);
}

void mutt_refresh(void)
{
    /* FIXME: UGGLY */
    extern ssize_t UngetCount;

    /* don't refresh when we are waiting for a child. */
    if (option(OPTKEEPQUIET))
        return;

    /* don't refresh in the middle of macros unless necessary */
    if (UngetCount && !option(OPTFORCEREFRESH))
        return;

    /* else */
    if (sidebar_w)
        wnoutrefresh(sidebar_w);
    if (main_w)
        wnoutrefresh(main_w);
    wrefresh(stdscr);
}

void mutt_endwin(const char *msg)
{
    if (!option(OPTNOCURSES)) {
        CLEARLINE(main_w, LINES - 1);

        if (sidebar_w) {
            delwin(sidebar_w);
            sidebar_w = NULL;
        }
#if 0
        if (main_w) {
            delwin(main_w);
            main_w = NULL;
        }
#endif

        mutt_refresh();
        endwin();
    }

    if (!m_strisempty(msg)) {
        puts(msg);
        fflush(stdout);
    }
}

void ui_layout_init(void)
{
    clear();
    main_w = newwin(LINES - 1, COLS, 0, 0);
    SETCOLOR(main_w, MT_COLOR_NORMAL);
    mutt_error   = mutt_curses_error;
    mutt_message = mutt_curses_message;
}

void ui_layout_resize(void)
{
    if (SigWinch) {
        int fd, rows = -1, cols = -1;
        struct winsize w;

        if ((fd = open("/dev/tty", O_RDONLY)) != -1) {
            if (ioctl(fd, TIOCGWINSZ, &w) != -1) {
                rows = w.ws_row;
                cols = w.ws_col;
            }
            close(fd);
        }
        if (rows <= 0) {
            rows = atoi(getenv("LINES") ?: "24");
        }
        if (cols <= 0) {
            cols = atoi(getenv("COLUMNS") ?: "80");
        }
        resizeterm(rows, cols);
        SigWinch = 0;
        /* force a real complete redraw. */
        clearok(main_w, true);
    }
}

WINDOW *ui_layout_sidebar_w(void)
{
    if (option(OPTMBOXPANE) && SidebarWidth > 1 && SidebarWidth < COLS)
    {
        int sw, sh;

        if (!sidebar_w) {
            sidebar_w = newwin(LINES - 1, SidebarWidth, 0, 0);
            wresize(main_w, LINES - 1, COLS - SidebarWidth);
            mvwin(main_w, 0, SidebarWidth);
        }

        getmaxyx(sidebar_w, sh, sw);

        if (sh != LINES - 1 || sw != SidebarWidth) {
            wresize(sidebar_w, LINES - 1, SidebarWidth);
            wresize(main_w, LINES - 1, COLS - SidebarWidth);
            mvwin(main_w, 0, SidebarWidth);
        }
    } else {
        if (sidebar_w) {
            delwin(sidebar_w);
            sidebar_w = NULL;
            wresize(main_w, LINES - 1, COLS);
            mvwin(main_w, 0, 0);
        }
    }
    return sidebar_w;
}
